from typing import Optional

from orchestrator.domain.base import ProductBlockModel
from orchestrator.types import SubscriptionLifecycle, strEnum
from gso.products.product_blocks.site \
    import SiteBlock, SiteBlockInactive, SiteBlockProvisioning
import ipaddress


class DeviceVendor(strEnum):
    juniper = "juniper"
    nokia = "nokia"


class DeviceRole(strEnum):
    p = "p"
    pe = "pe"
    amt = "amt"


class DeviceBlockInactive(ProductBlockModel,
                          lifecycle=[SubscriptionLifecycle.INITIAL],
                          product_block_name="DeviceBlock"):
    device_fqdn: Optional[str] = None
    device_ts_address: Optional[str] = None
    device_ts_port: Optional[int] = None
    device_access_via_ts: Optional[bool] = None
    device_lo_ipv4_address: Optional[ipaddress.IPv4Address] = None
    device_lo_ipv6_address: Optional[ipaddress.IPv6Address] = None
    device_lo_iso_address: Optional[str] = None
    device_si_ipv4_network: Optional[ipaddress.IPv4Network] = None
    device_ias_lt_ipv4_network: Optional[ipaddress.IPv4Network] = None
    device_ias_lt_ipv6_network: Optional[ipaddress.IPv6Network] = None
    device_vendor: Optional[DeviceVendor] = None
    device_role: Optional[DeviceRole] = None
    device_site: Optional[SiteBlockInactive]


class DeviceBlockProvisioning(DeviceBlockInactive,
                              lifecycle=[SubscriptionLifecycle.PROVISIONING]):
    device_fqdn: str
    device_ts_address: str
    device_ts_port: int
    device_access_via_ts: Optional[bool] = None
    device_lo_ipv4_address: Optional[ipaddress.IPv4Address] = None
    device_lo_ipv6_address: Optional[ipaddress.IPv6Address] = None
    device_lo_iso_address: Optional[str] = None
    device_si_ipv4_network: Optional[ipaddress.IPv4Network] = None
    device_ias_lt_ipv4_network: Optional[ipaddress.IPv4Network] = None
    device_ias_lt_ipv6_network: Optional[ipaddress.IPv6Network] = None
    device_vendor: Optional[DeviceVendor] = None
    device_role: Optional[DeviceRole] = None
    device_site: Optional[SiteBlockProvisioning]


class DeviceBlock(DeviceBlockProvisioning,
                  lifecycle=[SubscriptionLifecycle.ACTIVE]):
    device_fqdn: str
    device_ts_address: str
    device_ts_port: int
    device_access_via_ts: bool
    device_lo_ipv4_address: ipaddress.IPv4Address
    device_lo_ipv6_address: ipaddress.IPv6Address
    device_lo_iso_address: str
    device_si_ipv4_network: ipaddress.IPv4Network
    device_ias_lt_ipv4_network: ipaddress.IPv4Network
    device_ias_lt_ipv6_network: ipaddress.IPv6Network
    device_vendor: DeviceVendor
    device_role: DeviceRole
    device_site: SiteBlock
